/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "entity.h"
#include "assert/assert.h"
#include "util/memory.h"

entity_t ENTITIES[ENTITY_COUNT_MAX] = {0};

entitycallback_t ENTITY_CALLBACKS[ENTITY_TYPE_COUNT] = {
  {NULL}, // ENTITY_TYPE_NULL
  {playerNPCInit, playerNPCUpdate}, // ENTITY_TYPE_PLAYER
  {npcInit, npcUpdate} // ENTITY_TYPE_NPC
};

void entityInit(entity_t *entity, const entitytype_t type) {
  assertNotNull(entity, "Entity pointer cannot be NULL");
  assertTrue(type != ENTITY_TYPE_NULL, "Entity type NULL");
  assertTrue(type < ENTITY_TYPE_COUNT, "Entity type out of bounds");
  assertNotNull(
    ENTITY_CALLBACKS[type].init,
    "Entity type has no init callback"
  );
  
  memoryZero(entity, sizeof(entity_t));

  entity->type = type;
  ENTITY_CALLBACKS[type].init(entity);
}

void entityUpdate(entity_t *entity) {
  assertNotNull(entity, "Entity pointer cannot be NULL");
  assertTrue(entity->type != ENTITY_TYPE_NULL, "Entity type NULL");
  assertTrue(entity->type < ENTITY_TYPE_COUNT, "Entity type out of bounds");
  assertNotNull(
    ENTITY_CALLBACKS[entity->type].update,
    "Entity type has no update callback"
  );

  ENTITY_CALLBACKS[entity->type].update(entity);
}