/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "player.h"
#include "npc.h"

#define ENTITY_COUNT_MAX 32

typedef enum {
  ENTITY_DIR_NORTH = 0,
  ENTITY_DIR_SOUTH = 1,
  ENTITY_DIR_WEST = 2,
  ENTITY_DIR_EAST = 3,

  ENTITY_DIR_UP = ENTITY_DIR_NORTH,
  ENTITY_DIR_DOWN = ENTITY_DIR_SOUTH,
  ENTITY_DIR_LEFT = ENTITY_DIR_WEST,
  ENTITY_DIR_RIGHT = ENTITY_DIR_EAST,
} entitydir_t;

typedef enum {
  ENTITY_TYPE_NULL = 0,
  ENTITY_TYPE_PLAYER = 1,
  ENTITY_TYPE_NPC = 2,
} entitytype_t;
#define ENTITY_TYPE_COUNT 3

typedef struct _entity_t {
  uint32_t id;// Completely unique ID for this entity.
  float_t x, y;
  entitytype_t type;
  entitydir_t dir;

  union {
    npc_t npc;
    playerentity_t player;
  };
} entity_t;

typedef struct {
  void (*init) (entity_t *entity);
  void (*update) (entity_t *entity);
} entitycallback_t;

extern entity_t ENTITIES[ENTITY_COUNT_MAX];
extern entitycallback_t ENTITY_CALLBACKS[ENTITY_TYPE_COUNT];

/**
 * Initializes an entity with the given type.
 * 
 * @param entity Pointer to the entity to initialize.
 * @param type The type of the entity to initialize.
 */
void entityInit(entity_t *entity, const entitytype_t type);

/**
 * Updates the entity's state.
 * 
 * @param entity Pointer to the entity to update.
 */
void entityUpdate(entity_t *entity);