/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "render.h"
#include "display/draw/drawscene.h"
#include "display/draw/drawoverworld.h"
#include "assert/assert.h"

RenderTexture2D RENDER_SCREEN_TEXTURE;
Texture2D RENDER_TILEMAP_TEXTURE;

void renderInit(void) {
  InitWindow(
    RENDER_WIDTH * 3,
    RENDER_HEIGHT * 3,
    "Dusk Raylib Render"
  );
  SetTargetFPS(60);
  SetWindowState(
    FLAG_WINDOW_RESIZABLE |
    FLAG_WINDOW_HIGHDPI |
    FLAG_VSYNC_HINT
  );

  RENDER_SCREEN_TEXTURE = LoadRenderTexture(RENDER_WIDTH, RENDER_HEIGHT);
  RENDER_TILEMAP_TEXTURE = LoadTexture("../data/tilemap.png");

  drawOverworldInit();
}

void renderDraw(void) {
  // Draw the actual game.
  BeginTextureMode(RENDER_SCREEN_TEXTURE);
  ClearBackground(BLACK);
  
  drawScene();

  EndTextureMode();

  // Draw the render texture to the screen.
  BeginDrawing();
  ClearBackground(WHITE);
  // Keep aspect and center the render
  int32_t renderWidth, renderHeight, renderX, renderY;
  const int32_t width = GetScreenWidth();
  const int32_t height = GetScreenHeight();
  if(RENDER_WIDTH * height > RENDER_HEIGHT * width) {
    renderWidth = width;
    renderHeight = (RENDER_HEIGHT * width) / RENDER_WIDTH;
    renderX = 0;
    renderY = (height - renderHeight) / 2;
  } else {
    renderWidth = (RENDER_WIDTH * height) / RENDER_HEIGHT;
    renderHeight = height;
    renderX = (width - renderWidth) / 2;
    renderY = 0;
  }
  DrawTexturePro(
    RENDER_SCREEN_TEXTURE.texture,
    (Rectangle) { 0, 0, RENDER_WIDTH, -RENDER_HEIGHT },
    (Rectangle) { renderX, renderY, renderWidth, renderHeight },
    (Vector2) { 0, 0 },
    0.0f,
    WHITE
  );
  EndDrawing();
}

void renderDispose(void) {
  UnloadRenderTexture(RENDER_SCREEN_TEXTURE);
  CloseWindow();
}

bool_t renderShouldExit() {
  return WindowShouldClose();
}